package dev.fitko.fitconnect.api.domain.model.event;

import java.util.HashMap;
import java.util.Map;

import static dev.fitko.fitconnect.api.config.defaults.SchemaConfig.EVENTS_SCHEMA_PATH;

public enum Event {

    CREATE_SUBMISSION("create-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        public SubmissionState getState() {
            return SubmissionState.INCOMPLETE;
        }
    },

    CREATE_REPLY("create-reply") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        public SubmissionState getState() {
            return SubmissionState.INCOMPLETE;
        }
    },

    SUBMIT_SUBMISSION("submit-submission") {
        @Override
        public boolean hasAuthTags() {
            return true;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.SUBMITTED;
        }

    },

    SUBMIT_REPLY("submit-reply") {
        @Override
        public boolean hasAuthTags() {
            return true;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.SUBMITTED;
        }

    },

    NOTIFY_SUBMISSION("notify-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.NOTIFIED;
        }
    },

    NOTIFY_REPLY("notify-reply") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.NOTIFIED;
        }
    },

    FORWARD_SUBMISSION("forward-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.FORWARDED;
        }
    },

    ACCEPT_SUBMISSION("accept-submission") {
        @Override
        public boolean hasAuthTags() {
            return true;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.ACCEPTED;
        }
    },

    ACCEPT_REPLY("accept-reply") {
        @Override
        public boolean hasAuthTags() {
            return true;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.ACCEPTED;
        }
    },

    REJECT_SUBMISSION("reject-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.REJECTED;
        }
    },

    REJECT_REPLY("reject-reply") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.REJECTED;
        }
    },

    DELETE_SUBMISSION("delete-submission") {
        @Override
        public boolean hasAuthTags() {
            return false;
        }

        @Override
        public SubmissionState getState() {
            return SubmissionState.DELETED;
        }

    };

    private static final Map<String, Event> CONSTANTS = new HashMap<>();

    static {
        for (final Event c : values()) {
            CONSTANTS.put(c.getSchemaUri(), c);
        }
    }

    private final String name;

    Event(final String name) {
        this.name = name;
    }

    public static Event fromSchemaUri(final String schemaUri) {
        return CONSTANTS.get(schemaUri);
    }

    public String getSchemaUri() {
        return EVENTS_SCHEMA_PATH.getSchemaUri() + name;
    }

    public abstract boolean hasAuthTags();

    public abstract SubmissionState getState();
}
